<?php

namespace Drupal\Tests\homebox\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\homebox\Functional\HomeboxTestHelper;

/**
 * Tests for homebox javascript functionalities.
 *
 * @group homebox
 */
class HomeboxJavascriptTests extends WebDriverTestBase {

  /**
   * An admin user with all permissions.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * Basic user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * A homebox test helper instance.
   *
   * @var \Drupal\Tests\homebox\Functional\HomeboxTestHelper
   */
  protected $homeboxTestHelper;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'homebox',
    'block',
    'serialization',
    'layout_discovery',
    'path',
    'jquery_ui',
    'test_page_test',
    'user',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->homeboxTestHelper = new HomeboxTestHelper();
    $this->config('system.site')->set('page.front', '/test-page')->save();
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Waits for appearance of an element.
   *
   * @param string $type
   *   Expects 'xpath' or 'css'.
   * @param string|array $selector
   *   The selector to look for.
   */
  protected function waitForAppearance($type, $selector) {
    $page = $this->getSession()->getPage();
    return $page->waitFor(
      5,
      function () use ($type, $selector, $page) {
        return $page->find($type, $selector);
      }
    );
  }

  /**
   * Tests a homebox with the 'one column' Layout.
   */
  public function testHomeboxBlockOneColumn() {
    $drupalPoweredBlock = $this->homeboxTestHelper->createPseudoBlock('system_powered_by_block', '', 0, 'content', 1);
    $session = $this->assertSession();
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [$drupalPoweredBlock]);
    $this->drupalGet('/homebox/test');
    $session->pageTextContains('Powered by Drupal');
    $session->elementExists('css', '#homebox.layout--onecol');
    $session->elementExists('css', '#homebox-block-system_powered_by_block');
    $session->elementExists('css', 'div#homebox-block-system_powered_by_block > div.homebox-portlet-inner');
    $session->elementTextEquals('css', 'h3.portlet-header > span.portlet-title', 'Powered by Drupal');
    $session->elementExists('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper');
  }

  /**
   * Tests a homebox with the 'two column' Layout.
   */
  public function testHomeboxBlockTwoColumns() {
    $drupalPoweredBlock = $this->homeboxTestHelper->createPseudoBlock('system_powered_by_block', '', 0, 'first', 1);
    $drupalBrandingBlock = $this->homeboxTestHelper->createPseudoBlock('system_branding_block', '', 0, 'second', 1);
    $session = $this->assertSession();
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_twocol', [$drupalPoweredBlock, $drupalBrandingBlock]);
    $this->drupalGet('/homebox/test');
    $session->pageTextContains('Powered by Drupal');
    $session->pageTextContains('Site Branding');
    $session->elementExists('css', '#homebox.layout--twocol');
    $session->elementExists('css', '#homebox-block-system_powered_by_block');
    $session->elementExists('css', '#homebox-block-system_branding_block');
    $session->elementExists('css', 'div#homebox-block-system_branding_block > div.homebox-portlet-inner');
    $session->elementExists('css', 'div#homebox-block-system_powered_by_block > div.homebox-portlet-inner');
    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 2);
    $session->elementExists('css', '#region_first');
    $session->elementExists('css', '#region_second');
  }

  /**
   * Test Homebox add button functionality.
   */
  public function testHomeboxAddBlockButton() {
    /**
     * @var \Behat\Mink\Driver\Selenium2Driver $driver
     */
    $driver = $this->getSession()->getDriver();
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    $drupalPoweredBlock = $this->homeboxTestHelper->createPseudoBlock('system_powered_by_block', '', 0, 'content', 1);
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [$drupalPoweredBlock]);
    // Got to homebox dashboard:
    $this->drupalGet('/homebox/test');
    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 1);
    // Press the add Block button, and make sure
    // the block buttons are visible before clicking:
    $driver->click('//*[@id="edit-add-block-button"]');
    $driver->isVisible('//*[@id="edit-add-block-system-powered-by-block"]');
    // Add an aditional block:
    // @todo This is really weird:
    $driver->click('//*[@id="edit-add-block-system-powered-by-block"]');
    $page->pressButton('edit-add-block-system-powered-by-block');
    $this->waitForAppearance('css', 'nothing');

    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 2);
  }

  /**
   * Tests that the Homebox reset to defaults button works.
   */
  public function testHomeboxResetBlockButton() {
    // Note, that statusCodeEquals does not work in FunctionalJavascriptTests.
    $page = $this->getSession()->getPage();
    $session = $this->assertSession();
    $drupalPoweredBlock = $this->homeboxTestHelper->createPseudoBlock('system_powered_by_block', '', 0, 'content', 1);
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [$drupalPoweredBlock]);
    // Got to homebox dashboard:
    $this->drupalGet('/homebox/test');
    // Create a second rendered block:
    $page->pressButton('edit-add-block-button');
    $page->pressButton('edit-add-block-system-powered-by-block');
    $page->pressButton('edit-add-block-system-powered-by-block');
    $this->waitForAppearance('css', 'nothing');
    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 2);
    // Reset the blocks:
    $page->clickLink('edit-reset-block-button');
    $page->pressButton('edit-submit');
    // Check if only one homebox block exists now:
    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 1);
  }

  /**
   * Tests if deleting individual Blocks is possible.
   *
   * @todo Implement this, the click does not work.
   */
  public function todoTestDeletingIndividualBlock() {
    /**
     * @var \Behat\Mink\Driver\Selenium2Driver $driver
     */
    $driver = $this->getSession()->getDriver();
    // Note, that statusCodeEquals does not work in FunctionalJavascriptTests.
    $page = $this->getSession()->getPage();
    $session = $this->assertSession();
    $drupalPoweredBlock = $this->homeboxTestHelper->createPseudoBlock('system_powered_by_block', '', 0, 'content', 1);
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [$drupalPoweredBlock]);
    // Got to homebox dashboard:
    $this->drupalGet('/homebox/test');
    // Check if only one homebox block exists:
    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 1);
    $driver->click('//*[@id="homebox-block-system_powered_by_block"]/div/h3/a');
    // $page->pressButton('a.portlet-icon.portlet-close');
    $this->waitForAppearance('css', 'nothing');
    $session->elementsCount('css', 'div.homebox-draggable.js-form-wrapper.form-wrapper', 0);
  }

  /**
   * Test if a block is disabled, it won't show.
   *
   * @todo Implement this.
   */
  public function todoTestDisableBlock() {

  }

}
